--[[
	Addon communication plugin
]]--


--------------------------------------
-- Create plugin

local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("Comm", "AceComm-3.0", "AceSerializer-3.0")
if not plugin then return end

--------------------------------------
-- Locals
local prefix = "RW2"
local db
local throttle

--------------------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace("Comm", {profile={}})
	
	self.listeners = {}
	throttle = {}
	
	self:RegisterMessage("RW_RegisterCommListener")
end

function plugin:PluginEnable()
	self:RegisterComm(prefix)
	self:RegisterMessage("RW_RegisterCommListener")
	self:RegisterMessage("RW_UnregisterAllCommListeners")
	self:RegisterMessage("RW_SendComm")
end

function plugin:PluginDisable()
	self.listeners = nil
	throttle = nil
end

--------------------------------------
-- Comm handling

local function handleCom(self, sender, dist, success, ...)
	if success then
		local listener = ...
		if self.listeners[listener] then
			for module in pairs(self.listeners[listener]) do
				if module[listener] and type(module[listener]) == "function" then
					module[listener](module, sender, dist, select(2, ...))
				else
					if module.OnSync then
						module:OnSync(listener, sender, dist, select(2, ...))
					else
						--[===[@debug@
						error(("Module %q is missing handler for %q"):format(module.GetName and module:GetName() or module, listener))
						--@end-debug@]===]
					end
				end
			end
		end
	end
end

function plugin:OnCommReceived(_, message, dist, sender)
	handleCom(self, sender, dist, self:Deserialize(message))
end

--------------------------------------
-- Message handling

function plugin:RW_RegisterCommListener(message, module, listener)
	self.listeners[listener] = self.listeners[listener] or {}
	self.listeners[listener][module] = true
end

function plugin:RW_UnregisterListener(message, module, listener)
	if self.listeners[listener] then
		self.listener[listener][module] = nil
	end
end

function plugin:RW_UnregisterAllCommListeners(message, module)
	for listener, modules in pairs(self.listeners) do
		modules[module] = nil
	end
end

-- Send comm
function plugin:RW_SendComm(message, comMsg, dist, target, ...)
	-- Throttle the sending of the same message one 2 seconds
	local message = self:Serialize(comMsg, ...)
	if not throttle[message..dist..(target or "")] then
		-- If no dist is proviced, RAID will be used over PARTY, if none of them are true then no message is sent
		dist = dist or (GetRealNumRaidMembers() > 0 and "RAID" or (GetRealNumPartyMembers() > 0 and "PARTY" or nil))
		if dist ~= nil then
			self:SendCommMessage(prefix, message, dist, target)
			throttle[message..dist..(target or "")] = true
			self:ScheduleTimer("RemoveThrottle", 2, message..dist..(target or ""))
		end
	else
		--[===[@debug@
		error(("Sending of %q"..(target ~=nil and (" to "..target) or "") .." (spammed) "..(dist or "No Dist")):format(comMsg))
		--@end-debug@]===]
	end
end

function plugin:RemoveThrottle(message)
	throttle[message] = nil
end
